@@ -1,26 +1,59 @@
-import warnings
 from copy import copy
+from warnings import warn
 
-import CoolProp
 import CoolProp.CoolProp as CP
 import numpy as np
-from CoolProp.Plots import PropertyPlot
-from CoolProp.Plots.Common import interpolate_values_1d
 from plotly import graph_objects as go
-from bokeh.plotting import figure
-from bokeh.models import HoverTool, ColumnDataSource
 
 from . import Q_
 from .config.fluids import get_name, normalize_mix
-from .config.units import check_units, change_data_units
+from .config.units import check_units
 
 
 class State(CP.AbstractState):
-    """State class.
+    """A thermodynamic state.
+
+    .. note::
+        To create a state use State.define(...)
+
 
     This class is inherited from CP.AbstractState.
     Some extra functionality has been added.
-    To create a State see constructor .define().
+
+
+    Creates a state from fluid composition and two properties.
+    Properties should be in SI units, **kwargs can be passed
+    to change units.
+
+    Parameters
+    ----------
+    p : float
+        Pressure
+    T : float
+        Temperature
+    h : float
+        Enthalpy
+    s : float
+        Entropy
+    rho : float
+        Specific mass
+
+    fluid : dict
+        Dictionary with constituent and composition.
+        (e.g.: ({'Oxygen': 0.2096, 'Nitrogen': 0.7812, 'Argon': 0.0092})
+    EOS : string
+        String with HEOS or REFPROP
+
+    Returns
+    -------
+    state : State object
+
+    Examples
+    --------
+    >>> fluid = {'Oxygen': 0.2096, 'Nitrogen': 0.7812, 'Argon': 0.0092}
+    >>> s = State.define(fluid=fluid, p=101008, T=273, EOS='HEOS')
+    >>> s.rhomass()
+    1.2893965217814896
     """
 
     def __init__(self, EOS, _fluid):
@@ -47,8 +80,8 @@ class State(CP.AbstractState):
             other_fluid_rounded = {k: round(v, 3) for k, v in other.fluid.items()}
             if (
                 self_fluid_rounded == other_fluid_rounded
-                and np.allclose(self.p(), other.p())
-                and np.allclose(self.T(), other.T())
+                and np.allclose(self.p(), other.p(), rtol=1e-3)
+                and np.allclose(self.T(), other.T(), rtol=1e-3)
             ):
                 return True
         return False
@@ -62,37 +95,85 @@ class State(CP.AbstractState):
         return fluid_dict
 
     def gas_constant(self):
+        """Gas constant in joule / (mol kelvin).
+
+        Returns
+        -------
+        gas_constant : pint.Quantity
+            Gas constant (joule / (mol kelvin).
+        """
         return Q_(super().gas_constant(), "joule / (mol kelvin)")
 
     def molar_mass(self):
+        """Molar mass in kg/mol.
+
+        Returns
+        -------
+        molar_mass : pint.Quantity
+            Molar mass (kg/mol).
+        """
         return Q_(super().molar_mass(), "kg/mol")
 
     def T(self):
+        """Temperature in Kelvin.
+
+        Returns
+        -------
+        T : pint.Quantity
+            Temperature (Kelvin).
+        """
         return Q_(super().T(), "kelvin")
 
     def p(self):
+        """Pressure in Pascal.
+
+        Returns
+        -------
+        p : pint.Quantity
+            Pressure (pascal).
+        """
         return Q_(super().p(), "pascal")
 
     def cp(self):
+        """Specific heat at constant pressure joule/(kilogram kelvin).
+
+        Returns
+        -------
+        cp : pint.Quantity
+            Specific heat at constant pressure joule/(kilogram kelvin).
+        """
         return Q_(super().cpmass(), "joule/(kilogram kelvin)")
 
     def cv(self):
+        """Specific heat at constant volume joule/(kilogram kelvin).
+
+        Returns
+        -------
+        cv : pint.Quantity
+            Specific heat at constant volume joule/(kilogram kelvin).
+        """
         return Q_(super().cvmass(), "joule/(kilogram kelvin)")
 
     def h(self):
-        """Specific Enthalpy (per unit of mass)."""
+        """Specific Enthalpy (joule/kilogram).
+
+        Returns
+        -------
+        h : pint.Quantity
+            Enthalpy (joule/kilogram).
+        """
         return Q_(super().hmass(), "joule/kilogram")
 
     def s(self):
         """Specific entropy (per unit of mass)."""
         return Q_(super().smass(), "joule/(kelvin kilogram)")
 
-    def p_crit(self):
-        """ Critical Pressure in Pa """
+    def p_critical(self):
+        """Critical Pressure in Pa"""
         return Q_(super().p_critical(), "Pa")
 
-    def T_crit(self):
-        """ Critical Temperature in K """
+    def T_critical(self):
+        """Critical Temperature in K"""
         return Q_(super().T_critical(), "K")
 
     def rho(self):
@@ -112,10 +193,24 @@ class State(CP.AbstractState):
         return Q_(np.sqrt(self.first_partial_deriv(CP.iP, CP.iDmass, CP.iSmass)), "m/s")
 
     def viscosity(self):
+        """Viscosity in pascal second.
+
+        Returns
+        -------
+        viscosity : pint.Quantity
+            Viscosity (pascal second)
+        """
         return Q_(super().viscosity(), "pascal second")
 
     def kinematic_viscosity(self):
-        return (self.viscosity() / self.rho()).to("m**2/s")
+        """Kinematic viscosity in m²/s.
+
+        Returns
+        -------
+        kinematic_viscosity : pint.Quantity
+            Kinematic viscosity (m²/s)
+        """
+        return (self.viscosity() / self.rho()).to("m²/s")
 
     def dpdv_s(self):
         """
@@ -148,7 +243,13 @@ class State(CP.AbstractState):
         )
 
     def kv(self):
-        """Isentropic volume exponent (2.60)."""
+        """Isentropic volume exponent (dimensionless).
+
+        Returns
+        -------
+        kv : pint.Quantity
+            Isentropic volume exponent (dimensionless).
+        """
         return -(self.v() / self.p()) * self.dpdv_s()
 
     def dTdp_s(self):
@@ -161,14 +262,17 @@ class State(CP.AbstractState):
         )
 
     def kT(self):
-        """Isentropic temperature exponent (2.61)."""
+        """Isentropic temperature exponent (dimensionless).
+
+        Returns
+        -------
+        kT : pint.Quantity
+            Isentropic temperature exponent (dimensionless).
+        """
         return 1 / (1 - (self.p() / self.T()) * self.dTdp_s())
 
     def __reduce__(self):
-        # fluid_ = self.fluid
-        # kwargs = {k: v for k, v in self.init_args.items() if v is not None}
         kwargs = dict(p=self.p(), T=self.T(), fluid=self.fluid)
-        # kwargs['fluid'] = fluid_
         return self._rebuild, (self.__class__, kwargs)
 
     @staticmethod
@@ -207,10 +311,9 @@ class State(CP.AbstractState):
         rho : float
             Specific mass
 
-        fluid : dict or str
-            Dictionary with constituent and composition
+        fluid : dict
+            Dictionary with constituent and composition.
             (e.g.: ({'Oxygen': 0.2096, 'Nitrogen': 0.7812, 'Argon': 0.0092})
-            A pure fluid can be created with a string.
         EOS : string
             String with HEOS or REFPROP
 
@@ -224,10 +327,6 @@ class State(CP.AbstractState):
         >>> s = State.define(fluid=fluid, p=101008, T=273, EOS='HEOS')
         >>> s.rhomass()
         1.2893965217814896
-        >>> # pure fluid
-        >>> s = State.define(p=101008, T=273, fluid='CO2')
-        >>> s.rho()
-        1.9716931060214515
         """
         if fluid is None:
             raise TypeError("A fluid is required. Provide as fluid=dict(...)")
@@ -235,31 +334,20 @@ class State(CP.AbstractState):
         constituents = []
         molar_fractions = []
 
-        # if fluid is a string, consider pure fluid
-        try:
-            for k, v in fluid.items():
-                k = get_name(k)
-                constituents.append(k)
-                molar_fractions.append(v)
-                # create an adequate fluid string to cp.AbstractState
-            _fluid = "&".join(constituents)
-        except AttributeError:
-            # workaround https://github.com/CoolProp/CoolProp/issues/1544
-            # so we can build phase envelopes for pure substances
-            molar_fractions = [1 - 1e-15, 1e-15]
-            # First try to use the same fluid, if it does not work, use a mixture with a different fluid.
-            _fluid = f"{get_name(fluid)}&{get_name(fluid)}"
+        if len(fluid) < 2:
+            warn(
+                "Pure fluids are not fully supported and might break things (e.g. plot_phase_envelope"
+                "See https://github.com/CoolProp/CoolProp/issues/1544"
+            )
 
-        try:
-            state = cls(EOS, _fluid)
-        except ValueError:
-            mix_options = [get_name(f) for f in ("n2", "o2")]
-            if get_name(fluid) != mix_options[0]:
-                _fluid = f"{get_name(fluid)}&{mix_options[0]}"
-            else:
-                _fluid = f"{get_name(fluid)}&{mix_options[1]}"
-            state = cls(EOS, _fluid)
-            # TODO handle this with better error message, checking hmx.bnc
+        for k, v in fluid.items():
+            k = get_name(k)
+            constituents.append(k)
+            molar_fractions.append(v)
+            # create an adequate fluid string to cp.AbstractState
+        _fluid = "&".join(constituents)
+
+        state = cls(EOS, _fluid)
 
         normalize_mix(molar_fractions)
         state.set_mole_fractions(molar_fractions)
@@ -271,7 +359,9 @@ class State(CP.AbstractState):
 
         if isinstance(fluid, dict):
             if len(state.fluid) < len(fluid):
-                raise ValueError("You might have repeated components in the fluid dictionary.")
+                raise ValueError(
+                    "You might have repeated components in the fluid dictionary."
+                )
 
         state.update(**state.setup_args)
 
@@ -279,9 +369,15 @@ class State(CP.AbstractState):
 
     @check_units
     def update(
-        self, p=None, T=None, rho=None, h=None, s=None, **kwargs,
+        self,
+        p=None,
+        T=None,
+        rho=None,
+        h=None,
+        s=None,
+        **kwargs,
     ):
-        """Simple state update.
+        """Update the state.
 
         This method simplifies the state update. Only keyword arguments are
         required to update.
@@ -328,26 +424,29 @@ class State(CP.AbstractState):
     def plot_envelope(
         self, T_units="degK", p_units="Pa", dew_point_margin=20, fig=None, **kwargs
     ):
-        """Plot phase envelope
-            Plots the phase envelope and dew point limit.
-            Parameters
-            ----------
-            T_units : str
-                Temperature units.
-                Default is 'degK'.
-            p_units : str
-                Pressure units.
-                Default is 'Pa'.
-            dew_point_margin : float
-                Dew point margin.
-                Default is 20 degK (from API). Unit is the same as T_units.
-            fig : plotly.graph_objects.Figure, optional
-                The figure object with the rotor representation.
-            Returns
-            -------
-            fig : plotly.graph_objects.Figure
-                The figure object with the rotor representation.
-            """
+        """Plot phase envelope.
+
+        Plots the phase envelope and dew point limit.
+
+        Parameters
+        ----------
+        T_units : str
+            Temperature units.
+            Default is 'degK'.
+        p_units : str
+            Pressure units.
+            Default is 'Pa'.
+        dew_point_margin : float
+            Dew point margin.
+            Default is 20 degK (from API). Unit is the same as T_units.
+        fig : plotly.graph_objects.Figure, optional
+            The figure object with the rotor representation.
+
+        Returns
+        -------
+        fig : plotly.graph_objects.Figure
+            The figure object with the rotor representation.
+        """
         if fig is None:
             fig = go.Figure()
 
@@ -392,6 +491,15 @@ class State(CP.AbstractState):
             )
         )
 
+        fig.add_trace(
+            go.Scatter(
+                x=[self.T().to(T_units).m],
+                y=[self.p().to(p_units).m],
+                hovertemplate=hovertemplate,
+                name="State",
+            )
+        )
+
         fig.update_layout(
             xaxis=dict(title_text=f"Temperature ({T_units})"),
             yaxis=dict(
